﻿
Public Class Form1

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        EventsAreEnabled = False
        InitPanels()
        Load_INI()
        UpdateAllPanels()
        EnableDisablePanels()
        cmb_VectorizationType_Init()
        Me.Text = AppTitleAndVersion()
        EnsureFileInAppFolders()
        ReopenImage()
        SetParams()
        SetVectorizationParams()
        UpdateVectorsAndImage()
        RestartUpdateTimer()
        EventsAreEnabled = True
        Refresh()
        Opacity = 1
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        EventsAreEnabled = False
        Save_INI()
    End Sub

    Private Sub Form1_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Exit Sub
        LimitFormPosition(Me)
    End Sub

    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        If Not EventsAreEnabled Then Exit Sub
        UpdateVectorsAndImage()
    End Sub

    Private Sub SetDefaultFocus()
        Label_Resolution.Focus()
    End Sub

    Private Sub btn_LoadImage_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_LoadImage.ClickButtonArea
        EnsureFileInAppFolders()
        OpenImageDialog()
        Save_INI()
        ReopenImage()
        UpdateVectorsAndImage()
        SetParams()
        SetDefaultFocus()
    End Sub
    Private Sub btn_ReloadImage_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_ReloadImage.ClickButtonArea
        ReopenImage()
        SetParams()
        SetDefaultFocus()
    End Sub
    Private Sub btn_Rosenfeld_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_Rosenfeld.ClickButtonArea
        If btn_Rosenfeld.Checked Then btn_Rosenfeld2.Checked = False
        ReopenImage()
        UpdateVectorsAndImage()
        SetDefaultFocus()
        RestartUpdateTimer()
    End Sub
    Private Sub btn_Rosenfeld2_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_Rosenfeld2.ClickButtonArea
        If btn_Rosenfeld2.Checked Then btn_Rosenfeld.Checked = False
        ReopenImage()
        UpdateVectorsAndImage()
        SetDefaultFocus()
        RestartUpdateTimer()
    End Sub

    ' ========================================================================================================
    '   VECTORIZATION COMMANDS
    ' ========================================================================================================
    Private Sub btn_ViewVectorized_CheckedChanged(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_ViewVectorized.CheckedChanged
        UpdateVectorsAndImage()
        EnableDisablePanels()
    End Sub

    Private Sub EnableDisablePanels()
        If btn_ViewVectorized.Checked Then
            lbl_Threshold.Enabled = True
            txt_Threshold.Enabled = True
            chk_WeightedDw.Enabled = True
            cmb_VectorizationType.Enabled = True
            cmb_VectorizationType.BackColor = Color.Cornsilk
            GroupBox4.Enabled = True
            GroupBox5.Enabled = True
        Else
            lbl_Threshold.Enabled = False
            txt_Threshold.Enabled = False
            chk_WeightedDw.Enabled = False
            cmb_VectorizationType.Enabled = False
            cmb_VectorizationType.BackColor = Color.FromArgb(210, 210, 210)
            GroupBox4.Enabled = False
            GroupBox5.Enabled = False
        End If
    End Sub

    Private Sub btn_CreateGcode_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_CreateGcode.ClickButtonArea
        Gcode_FromVectorList(ImageFile)
        Gcode_WriteFile()
        SetDefaultFocus()
    End Sub

    Private Sub btn_TestHexagons_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_TestHexagons.ClickButtonArea
        CreateTestExagons(Application.StartupPath & "\media\Test_Hexagons.gc")
    End Sub

    Private Sub OpenImageDialog()
        ImageFile = PlatformAdjustedFileName(ImageFile, Application.StartupPath & "/../media/")
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.Filter = "Image files (*.jpeg;*.gif;*.png;*.bmp;*.tiff;*.emf;*.wmf)|*.jpeg;*.jpg;*.gif;*.png;*.bmp;*.tiff;*.emf;*.wmf;*.tif|All files (*.*)|*.*"
        If IO.File.Exists(ImageFile) Then
            ofd.InitialDirectory = IO.Path.GetDirectoryName(ImageFile)
            ofd.FileName = ImageFile
        Else
            ofd.InitialDirectory = PlatformAdjustedFileName(Application.StartupPath & "\Media")
            ofd.FileName = ""
        End If
        ImageFile = PlatformAdjustedFileName(ImageFile)
        ofd.ShowDialog()
        ImageFile = ofd.FileName
    End Sub

    Private Sub ReopenImage()
        OpenImage(Pbox1, txt_Resolution.NumericValueInteger, CSng(txt_Blur.NumericValue))
        If btn_Rosenfeld.Checked Then ImageToRosenfeld()
        If btn_Rosenfeld2.Checked Then ImageToRosenfeld_2()
    End Sub

    Private Sub txt_Resolution_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                            Handles txt_Resolution.TextChanged
        If Not EventsAreEnabled Then Return
        If txt_Resolution.NumericValueInteger < 32 Then Return
        ReopenImage()
        RestartUpdateTimer()
    End Sub

    Private Sub txt_Blur_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                            Handles txt_Blur.TextChanged
        If Not EventsAreEnabled Then Return
        ReopenImage()
        RestartUpdateTimer()
    End Sub

    Private Sub Params_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                            Handles txt_Resolution.LostFocus, _
                                                                                  txt_Blur.LostFocus, _
                                                                                  txt_Threshold.LostFocus, _
                                                                                  chk_WeightedDw.LostFocus, _
                                                                                  txt_Width.LostFocus, _
                                                                                  txt_Height.LostFocus, _
                                                                                  chk_UseImageAspect.LostFocus, _
                                                                                  chk_UseImageSize.LostFocus, _
                                                                                  txt_Up.LostFocus, _
                                                                                  txt_Down.LostFocus, _
                                                                                  txt_ToolSize.LostFocus, _
                                                                                  txt_Feed.LostFocus, _
                                                                                  txt_Rapids.LostFocus, _
                                                                                  txt_Acceleration.LostFocus
        If Not EventsAreEnabled Then Return
        Save_INI()
    End Sub

    Private Sub Params_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                            Handles chk_UseImageAspect.CheckedChanged, _
                                                                                    chk_UseImageSize.CheckedChanged, _
                                                                                    txt_Up.TextChanged, _
                                                                                    txt_ToolSize.TextChanged, _
                                                                                    txt_Down.TextChanged, _
                                                                                    txt_Feed.TextChanged, _
                                                                                    txt_Rapids.TextChanged, _
                                                                                    txt_Acceleration.TextChanged
        If Not EventsAreEnabled Then Return
        SetParams()
    End Sub

    Private Sub SetParams()
        Gcode_UseImageAspect = chk_UseImageAspect.Checked
        Gcode_UseImageSize = chk_UseImageSize.Checked
        If chk_UseImageSize.Checked Then
            chk_UseImageAspect.Checked = True
            txt_Width.NumericValue = GetOutWidth()
            txt_Height.NumericValue = GetOutHeight()
            chk_UseImageAspect.Enabled = False
            txt_Width.Enabled = False
            txt_Height.Enabled = False
        Else
            SetGcodeWidth(txt_Width.NumericValueInteger)
            SetGcodeHeight(txt_Height.NumericValueInteger)
            chk_UseImageAspect.Enabled = True
            txt_Width.Enabled = True
            txt_Height.Enabled = True
        End If

        Gcode_Zup = CSng(txt_Up.NumericValue)
        Gcode_Zdown = CSng(txt_Down.NumericValue)
        Gcode_ToolSize = CSng(txt_ToolSize.NumericValue)
        Gcode_Feed = txt_Feed.NumericValueInteger
        Gcode_Rapids = txt_Rapids.NumericValueInteger
        Gcode_Acceleration = txt_Acceleration.NumericValueInteger
        '
        RestartUpdateTimer()
    End Sub


    Private Sub VectorizationParams_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                              Handles txt_Threshold.TextChanged, _
                                                                                      chk_WeightedDw.CheckedChanged

        If Not EventsAreEnabled Then Return
        If TypeOf (sender) Is TextBox Then
            CType(sender, TextBox).Refresh()
        End If
        If TypeOf (sender) Is CheckBox Then
            CType(sender, CheckBox).Refresh()
        End If
        SetVectorizationParams()
        UpdateVectorsAndImage()
        RestartUpdateTimer()
    End Sub

    ' ======================================================================
    '   COMBO cmb_VectorizationType
    ' ======================================================================
    Private Sub cmb_VectorizationType_Init()
        cmb_VectorizationType.ItemHeight = 12
    End Sub
    Private Sub cmb_VectorizationType_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_VectorizationType.DropDown
        cmb_VectorizationType.ItemHeight = 16
    End Sub
    Private Sub cmb_VectorizationType_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_VectorizationType.DropDownClosed
        cmb_VectorizationType_Init()
    End Sub
    Private Sub cmb_VectorizationType_SelectionChangeCommitted(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_VectorizationType.SelectionChangeCommitted
        SetVectorizationParams()
        UpdateVectorsAndImage()
        RestartUpdateTimer()
    End Sub

    Private Sub SetVectorizationParams()
        Gcode_WeightedDw = chk_WeightedDw.Checked
        VectorizationType = cmb_VectorizationType.SelectedIndex
    End Sub

    ' TODO
    Private Sub UpdateVectorsAndImage()
        If btn_ViewVectorized.Checked Then
            ' -------------------------------------------------------------
            ' Dim sw As Stopwatch = New Stopwatch
            ' sw.Start()
            ' -------------------------------------------------------------
            If MainBitmap Is Nothing Then Return
            ' -------------------------------------------------------------
            CreateVectors(txt_Threshold.NumericValueInteger)
            ' -------------------------------------------------------------
            Dim aspect As Double = MainBitmap.Width / MainBitmap.Height
            Dim w, h As Int32
            w = Pbox1.ClientRectangle.Width
            h = Pbox1.ClientRectangle.Height
            If h * aspect > w Then
                h = CInt(w / aspect)
            Else
                w = CInt(h * aspect)
            End If
            Pbox1.BackgroundImage = VectorsToImage(w, h)
            RestartUpdateTimer()
            ' -------------------------------------------------------------
            ' Text = sw.Elapsed.TotalMilliseconds.ToString
            ' -------------------------------------------------------------
        Else
            Pbox1.BackgroundImage = MainBitmap
        End If
        Pbox1.Refresh()
    End Sub

    ' ========================================================================================================
    '   GCODE commands
    ' ========================================================================================================
    Private Sub txt_Width_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                        Handles txt_Width.TextChanged
        If Not EventsAreEnabled Then Return
        SetGcodeWidth(txt_Width.NumericValueInteger)
        EventsAreEnabled = False
        txt_Height.NumericValue = GetOutHeight()
        txt_Height.Refresh()
        EventsAreEnabled = True
        RestartUpdateTimer()
    End Sub
    Private Sub txt_Height_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                         Handles txt_Height.TextChanged
        If Not EventsAreEnabled Then Return
        SetGcodeHeight(txt_Height.NumericValueInteger)
        EventsAreEnabled = False
        txt_Width.NumericValue = GetOutWidth()
        txt_Width.Refresh()
        EventsAreEnabled = True
        RestartUpdateTimer()
    End Sub


    ' ========================================================================================================
    '   Update timer
    ' ========================================================================================================
    Private WithEvents UpdateTimer As Timer = New Timer
    Private Sub RestartUpdateTimer()
        UpdateTimer.Interval = 100
        UpdateTimer.Stop()
        UpdateTimer.Start()
    End Sub
    Private Sub UpdateTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UpdateTimer.Tick
        UpdateTimer.Stop()

        ' TODO REMOVE THIS
        'CreateVectors(txt_Threshold.NumericValueInteger)

        Gcode_FromVectorList(ImageFile)
        lbl_Time.Text = Gcode_GetTotalTime()
    End Sub



    ' ===================================================================================================
    '  PANELS 
    ' ===================================================================================================
    Private PanelHeight(5) As Int32

    Private Sub InitPanels()
        PanelHeight(1) = GroupBox1.Height
        PanelHeight(2) = GroupBox2.Height
        PanelHeight(3) = GroupBox3.Height
        PanelHeight(4) = GroupBox4.Height
        PanelHeight(5) = GroupBox5.Height
    End Sub

    Private Sub GroupBox_Click(ByVal sender As Object, ByVal e As EventArgs) Handles GroupBox1.Click, _
                                                                                     GroupBox2.Click, _
                                                                                     GroupBox3.Click, _
                                                                                     GroupBox4.Click, _
                                                                                     GroupBox5.Click
        Dim gbox As GroupBox = CType(sender, GroupBox)
        If gbox.PointToClient(MousePosition).Y > 20 Then Return
        ToggleGboxHeight(gbox)
        UpdateAllPanels()
    End Sub

    Private Sub ToggleGboxHeight(ByVal gbox As GroupBox)
        Dim i As Int32 = CInt(Val(gbox.Name.Substring(gbox.Name.Length - 1)))
        If gbox.Height > 20 Then
            gbox.Height = 16
        Else
            gbox.Height = PanelHeight(i)
        End If
    End Sub

    Private Sub SetGboxHeight(ByVal gbox As GroupBox)
        Dim i As Int32 = CInt(Val(gbox.Name.Substring(gbox.Name.Length - 1)))
        If gbox.Height <= 20 Then
            gbox.Height = 16
        Else
            gbox.Height = PanelHeight(i)
        End If
    End Sub

    Private Sub UpdateAllPanels()
        ' --------------------------------------- Use this method if some GBOX is not located on the form
        'GroupBox4.Parent = Me
        'GroupBox4.Left = GroupBox1.Left
        'GroupBox5.Parent = Me
        'GroupBox5.Left = GroupBox1.Left
        ' --------------------------------------- Ensure the correct height also if INI is not ok
        SetGboxHeight(GroupBox1)
        SetGboxHeight(GroupBox2)
        SetGboxHeight(GroupBox3)
        SetGboxHeight(GroupBox4)
        SetGboxHeight(GroupBox5)
        ' --------------------------------------- Set the vertical positions
        GroupBox2.Top = GroupBox1.Bottom + 6
        GroupBox3.Top = GroupBox2.Bottom + 6
        GroupBox4.Top = GroupBox3.Bottom + 6
        GroupBox5.Top = GroupBox4.Bottom + 6
        ' --------------------------------------- Set the Form minimum height
        Dim topSize As Int32 = Me.Size.Height - Me.ClientSize.Height
        Dim MinHeight As Int32 = GroupBox5.Bottom + topSize + 5
        If MinHeight < 480 Then MinHeight = 480
        Me.MinimumSize = New Size(Me.MinimumSize.Width, MinHeight)
    End Sub

End Class