﻿
Imports System.Collections.Generic

Module Module_Vectorize

    Friend VectorizationType As Int32

    Friend Class Vector
        Friend points As List(Of PointF)
        'Friend z As Single
        'Friend down As Boolean
        Friend Sub New()
            points = New List(Of PointF)
        End Sub
    End Class

    Friend Class VectorList
        Friend vectors As List(Of Vector)
        Friend Sub New()
            vectors = New List(Of Vector)
        End Sub
    End Class

    Friend VecList As VectorList = New VectorList
    Private ba(-1, -1) As Boolean
    Private sizeX As Integer
    Private sizeY As Integer

    ' ========================================================================================================
    '  Vectors to Image
    ' ========================================================================================================
    Friend Function VectorsToImage(ByVal w As Int32, ByVal h As Int32) As Bitmap
        '
        If w < 1 Or h < 1 Then Return Nothing
        Dim bmp As Bitmap = New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(bmp)
        g.SmoothingMode = Drawing2D.SmoothingMode.HighSpeed
        g.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic
        g.Clear(Color.White)
        If VecList Is Nothing Then Return bmp
        '
        Dim oldx As Single
        Dim oldy As Single
        Dim x, y As Single
        '
        For Each v As Vector In VecList.vectors
            oldx = -999
            For Each p As PointF In v.points
                x = p.X * w
                y = p.Y * h
                If oldx >= 0 Then
                    g.DrawLine(Pens.Black, oldx, oldy, x, y)
                End If
                oldx = x
                oldy = y
            Next
        Next
        Return bmp
    End Function

    ' TODO
    ' ========================================================================================================
    '  CreateVectors
    ' ========================================================================================================
    Friend Sub CreateVectors(ByVal Threshold As Int32)
        '
        'Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()
        '
        If MainBitmap Is Nothing Then Return
        sizeX = MainBitmap.Width
        sizeY = MainBitmap.Height
        ' ----------------------------------------------------------------- BITMAP TO BYNARY ARRAY
        Bitmap_to_BinaryArray(MainBitmap, ba, Threshold)
        ' ----------------------------------------------------------------- 
        'Form1.Text = (sw1.ElapsedMilliseconds).ToString
        'sw1.Reset()
        'sw1.Start()
        ' ----------------------------------------------------------------- CREATE VECTORS
        VecList = New VectorList
        Select Case VectorizationType
            Case 0 : CreateVectors_Vertical()
            Case 1 : CreateVectors_Horizontal()
            Case 2 : CreateVectors_FollowBorders()
        End Select
        ' ----------------------------------------------------------------- CONNECT AND SIMPLIFY
        ConnectVectors()
        'SimplifyVectors()
        OptimizeRapids(0, 1)

        ' ----------------------------------------------------------------- 
        'Form1.Text &= " " & (sw1.ElapsedMilliseconds).ToString
        'sw1.Reset()
        'sw1.Start()
    End Sub

    Private Sub CreateVectors_Vertical()
        For x As Int32 = 1 To sizeX - 2
            Dim vec As Vector = New Vector
            For y As Int32 = 1 To sizeY - 2
                If ba(x, y) Then
                    vec.points.Add(New PointF(CSng(x / sizeX), CSng(y / sizeY)))
                Else
                    If vec.points.Count > 0 Then
                        VecList.vectors.Add(vec)
                        vec = New Vector
                    End If
                End If
            Next
            If vec.points.Count > 0 Then
                VecList.vectors.Add(vec)
                vec = New Vector
            End If
        Next
    End Sub
    Private Sub CreateVectors_Horizontal()
        For y As Int32 = 1 To sizeY - 2
            Dim vec As Vector = New Vector
            For x As Int32 = 1 To sizeX - 2
                If ba(x, y) Then
                    vec.points.Add(New PointF(CSng(x / sizeX), CSng(y / sizeY)))
                Else
                    If vec.points.Count > 0 Then
                        VecList.vectors.Add(vec)
                        vec = New Vector
                    End If
                End If
            Next
            If vec.points.Count > 0 Then
                VecList.vectors.Add(vec)
                vec = New Vector
            End If
        Next
    End Sub

    Private Sub CreateVectors_FollowBorders()
        For x As Int32 = 1 To sizeX - 2
            For y As Int32 = 1 To sizeY - 2
                If ba(x, y) Then
                    FollowNewVector(x, y)
                End If
            Next
        Next
    End Sub

    Private Sub FollowNewVector(ByVal x As Int32, ByVal y As Int32)
        Dim vec As Vector = New Vector
        vec.points.Add(New PointF(CSng(x / sizeX), CSng(y / sizeY)))
        ba(x, y) = False
        While FindNextPoint(x, y)
            vec.points.Add(New PointF(CSng(x / sizeX), CSng(y / sizeY)))
            ba(x, y) = False
        End While
        VecList.vectors.Add(vec)
    End Sub

    Private Function FindNextPoint(ByRef x As Int32, ByRef y As Int32) As Boolean
        x += 1
        If ba(x, y) Then Return True
        y -= 1
        If ba(x, y) Then Return True
        x -= 1
        If ba(x, y) Then Return True
        x -= 1
        If ba(x, y) Then Return True
        y += 1
        If ba(x, y) Then Return True
        y += 1
        If ba(x, y) Then Return True
        x += 1
        If ba(x, y) Then Return True
        x += 1
        If ba(x, y) Then Return True
        Return False
    End Function

    Private Sub ConnectVectors()
        Dim i1 As Int32
        Dim i2 As Int32
        i1 = 0
        While i1 < VecList.vectors.Count - 1
            Dim StartPoint1 As PointF = VecList.vectors(i1).points(0)
            Dim EndPoint1 As PointF = VecList.vectors(i1).points(VecList.vectors(i1).points.Count - 1)
            i2 = i1 + 1
            While i2 < VecList.vectors.Count
                Dim StartPoint2 As PointF = VecList.vectors(i2).points(0)
                Dim EndPoint2 As PointF = VecList.vectors(i2).points(VecList.vectors(i2).points.Count - 1)
                If AdiacentPoints(EndPoint1, StartPoint2) Then
                    VecList.vectors(i1).points.AddRange(VecList.vectors(i2).points)
                    VecList.vectors.Remove(VecList.vectors(i2))
                    i1 -= 1
                    Exit While
                End If

                If AdiacentPoints(EndPoint1, EndPoint2) Then
                    VecList.vectors(i2).points.Reverse()
                    VecList.vectors(i1).points.AddRange(VecList.vectors(i2).points)
                    VecList.vectors.Remove(VecList.vectors(i2))
                    i1 -= 1
                    Exit While
                End If

                If AdiacentPoints(StartPoint1, StartPoint2) Then
                    VecList.vectors(i1).points.Reverse()
                    VecList.vectors(i1).points.AddRange(VecList.vectors(i2).points)
                    VecList.vectors.Remove(VecList.vectors(i2))
                    i1 -= 1
                    Exit While
                End If

                If AdiacentPoints(StartPoint1, EndPoint2) Then
                    VecList.vectors(i1).points.Reverse()
                    VecList.vectors(i2).points.Reverse()
                    VecList.vectors(i1).points.AddRange(VecList.vectors(i2).points)
                    VecList.vectors.Remove(VecList.vectors(i2))
                    i1 -= 1
                    Exit While
                End If
                i2 += 1
            End While
            i1 += 1
        End While
    End Sub

    Private Function AdiacentPoints(ByVal p1 As PointF, ByVal p2 As PointF) As Boolean
        If Math.Abs(p1.X - p2.X) < 0.005 And Math.Abs(p1.Y - p2.Y) <= 0.005 Then
            Return True
        Else
            Return False
        End If
    End Function

    'Private Sub SimplifyVectors()
    '    ' ----------------------------------------------------------- remove by distance
    '    'Dim oldpoint As Point
    '    'Dim i As Int32
    '    'For Each vec As Vector In vl.vectors
    '    '    oldpoint.X = 0
    '    '    oldpoint.Y = 0
    '    '    i = 1
    '    '    While i < vec.points.Count - 1
    '    '        Dim p As Point = vec.points(i)
    '    '        If PointDistance(oldpoint, p) < 5 Then
    '    '            vec.points.Remove(p)
    '    '            i -= 1
    '    '        Else
    '    '            oldpoint = p
    '    '        End If
    '    '        i += 1
    '    '    End While
    '    'Next

    '    ' ----------------------------------------------------------- remove one each two
    '    'Dim i As Int32
    '    'For Each vec As Vector In vl.vectors
    '    '    i = 1
    '    '    While i < vec.points.Count - 1
    '    '        Dim p As Point = vec.points(i)
    '    '        If i Mod (2) <> 0 Then
    '    '            vec.points.Remove(p)
    '    '        End If
    '    '        i += 1
    '    '    End While
    '    'Next
    'End Sub

    Private Sub OptimizeRapids(Optional ByVal x As Single = 0, Optional ByVal y As Single = 0)
        Dim dummy As Vector
        For i As Int32 = 0 To VecList.vectors.Count - 1
            Dim ix As Int32 = FindNearestVectorIndex(x, y, i)
            If ix < Int32.MaxValue Then
                If ix < 0 Then
                    ix = -ix - 1
                    VecList.vectors(ix).points.Reverse()
                End If
                dummy = VecList.vectors(i)
                VecList.vectors(i) = VecList.vectors(ix)
                VecList.vectors(ix) = dummy
                With VecList.vectors(i).points(VecList.vectors(i).points.Count - 1)
                    x = .X
                    y = .Y
                End With
            End If
        Next
    End Sub

    Private Function FindNearestVectorIndex(ByVal x As Single, ByVal y As Single, ByVal startIndex As Int32) As Int32
        Dim p As PointF = New PointF(x, y)
        Dim BestIndex As Int32 = Int32.MaxValue
        Dim BestDist As Double = Double.MaxValue
        Dim dist As Double
        For i As Int32 = startIndex To VecList.vectors.Count - 1
            Dim vec As Vector = VecList.vectors(i)
            ' ------------------------------------------------------------ compare start points
            dist = PointDistance(p, vec.points(0))
            If dist < BestDist Then
                BestDist = dist
                BestIndex = i
            End If
            ' ------------------------------------------------------------ compare end points
            dist = PointDistance(p, vec.points(vec.points.Count - 1))
            If dist < BestDist Then
                BestDist = dist
                BestIndex = -1 - i ' for end points return a negative index starting with -1
            End If
        Next
        Return BestIndex
    End Function

    Private Function PointDistance(ByVal p1 As PointF, ByVal p2 As PointF) As Double
        Return Math.Sqrt((p1.X - p2.X) ^ 2 + (p1.Y - p2.Y) ^ 2)
    End Function


End Module
